@@ -23,6 +23,9 @@ wt_y = [6151447, 6150891, 6150335, 6149779, 6149224, 6148668, 6148112, 6147556,
         6151447, 6150891, 6150335, 6149779, 6149224, 6148668, 6148112, 6147556]
 wt9_x = np.array(wt_x)[[0, 1, 2, 8, 9, 10, 16, 17, 18]]
 wt9_y = np.array(wt_y)[[0, 1, 2, 8, 9, 10, 16, 17, 18]]
+i16 = [0, 1, 2, 3, 8, 9, 10, 11, 16, 17, 18, 19, 24, 25, 26, 27]
+wt16_x = np.array(wt_x)[i16]
+wt16_y = np.array(wt_y)[i16]
 
 
 power_curve = np.array([[3.0, 0.0],
@@ -47,7 +50,7 @@ power_curve = np.array([[3.0, 0.0],
                         [22.0, 2000.0],
                         [23.0, 2000.0],
                         [24.0, 2000.0],
-                        [25.0, 2000.0]])
+                        [25.0, 2000.0]]) * [1, 1000]
 ct_curve = np.array([[3.0, 0.0],
                      [4.0, 0.818],
                      [5.0, 0.806],
@@ -76,7 +79,7 @@ ct_curve = np.array([[3.0, 0.0],
 class V80(OneTypeWindTurbines):
     def __init__(self):
         OneTypeWindTurbines.__init__(self, 'V80', diameter=80, hub_height=70,
-                                     ct_func=self._ct, power_func=self._power, power_unit='kW')
+                                     ct_func=self._ct, power_func=self._power, power_unit='W')
 
     def _ct(self, u):
         return np.interp(u, ct_curve[:, 0], ct_curve[:, 1])
